"use strict";

/**
 * Delete everything in a `range`.
 *
 * @param {State} state
 * @param {Selection} range
 * @return {State}
 */

function deleteAtRange(state, range) {
  // If the range is collapsed, there's nothing to delete.
  if (range.isCollapsed) return state;

  // Make sure the children exist.
  var document = state.document;
  var startKey = range.startKey;
  var startOffset = range.startOffset;
  var endKey = range.endKey;
  var endOffset = range.endOffset;

  document.assertDescendant(startKey);
  document.assertDescendant(endKey);

  // If the start and end nodes are the same, just remove characters.
  if (startKey == endKey) {
    var text = document.getDescendant(startKey);
    text = text.removeCharacters(startOffset, endOffset);
    document = document.updateDescendant(text);
    return document.normalize();
  }

  // Split the blocks and determine the edge boundaries.
  var start = range.collapseToStart();
  var end = range.collapseToEnd();
  var ancestor = document.getCommonAncestor(startKey, endKey);
  var isAncestor = ancestor == document;

  ancestor = ancestor.splitBlockAtRange(start, Infinity);
  ancestor = ancestor.splitBlockAtRange(end, Infinity);

  var startText = ancestor.getDescendant(startKey);
  var startEdgeText = ancestor.getNextText(startKey);

  var endText = ancestor.getNextText(endKey);
  var endEdgeText = ancestor.getDescendant(endKey);

  // Remove the new blocks inside the edges.
  var startEdgeBlock = ancestor.getFurthestBlock(startEdgeText);
  var endEdgeBlock = ancestor.getFurthestBlock(endEdgeText);

  var nodes = ancestor.nodes.takeUntil(function (n) {
    return n == startEdgeBlock;
  }).concat(ancestor.nodes.skipUntil(function (n) {
    return n == endEdgeBlock;
  }).rest());

  ancestor = ancestor.merge({ nodes: nodes });

  // Take the end edge's split text and move it to the start edge.
  var startBlock = ancestor.getClosestBlock(startText);
  var endChild = ancestor.getFurthestInline(endText) || endText;

  var startNodes = startBlock.nodes.push(endChild);
  startBlock = startBlock.merge({ nodes: startNodes });
  ancestor = ancestor.updateDescendant(startBlock);

  // While the end child is an only child, remove the block it's in.
  var endParent = ancestor.getClosestBlock(endChild);

  while (endParent && endParent.nodes.size == 1) {
    endChild = endParent;
    endParent = ancestor.getClosestBlock(endParent);
  }

  ancestor = ancestor.removeDescendant(endChild);

  // Update the document.
  document = isAncestor ? ancestor : document.updateDescendant(ancestor);

  // Normalize the adjacent text nodes.
  return document.normalize();
}